"""
Global constants of application
"""

# Real time delay between two executions of application (ethical scraping)
MIN_SECONDS_REQUEST: float = 10.0


# Seconds to wait between requests to same endpoint (ethical scraping)
WAIT_BETWEEN_PINGS: int = 1
